//  Unit of TSP Solver
//  Created by GrMikeD, 2000-2003
//  http://kickme.to/GrMikeD
//  GrMikeD@Freemail.gr
unit Graph;

interface

type
  TGraph = class
  private
    fN: Integer;
    fValues: array of array of Integer;
    procedure CreateValues;
    procedure DestroyValues;
    function GetSize: Integer;
    procedure SetSize(const Value: Integer);
    function ReadFileLn(var F: TextFile; out Ln: string): Boolean;
    function AddRow(r: string; Col: integer): Boolean;
  public
    constructor Create(const N: Integer);
    destructor Destroy; override;
    function LoadValuesFromFile(const FName: string): integer;
    function SaveValuesToFile(const FName: string): integer;
    procedure LoadRandomValues(const IsConnected: Boolean = False);
    function GetValue(const i, j: Integer): Integer;
  published
    property Size: Integer read GetSize write SetSize;
  end;


implementation

uses
  SysUtils, Classes, Error;

const
  MAX_NUM = 5000;
  RES_NUM = 21;
  INC_NUM = 5;

{ TGraph }

constructor TGraph.Create(const N: Integer);
begin
  fN:= N;
  CreateValues;
end;

destructor TGraph.Destroy;
begin
  DestroyValues;
  inherited Destroy;
end;

procedure TGraph.CreateValues;
begin
  SetLength(fValues, fN, fN);
end;

procedure TGraph.DestroyValues;
begin
  fValues:= nil;
end;

procedure TGraph.LoadRandomValues(const IsConnected: Boolean = False);
var
  i, j: Integer;
  Value: Integer;
begin
  Randomize;
  for i:= 0 to fN - 1 do
    for j:= 0 to fN - 1 do
    begin
      if (j = i) then
      begin
        fValues[i, j]:= -1;
        Continue;
      end;
      if (j > i) then
      begin
        repeat
          Value:= Random(MAX_NUM - INC_NUM);
        until (Value <> 0);
        if ((not IsConnected) and (Value mod RES_NUM = 0)) then
          Value:= -1
        else
          Inc(Value, INC_NUM);
        fValues[i, j]:= Value;
        fValues[j, i]:= Value;
      end;
    end;
end;

// ugly but quick way!
function TGraph.AddRow(r: string; Col: integer): Boolean;
var
  List: TStringList;
  i, n: Integer;
  s: string;

function IsNum(const s:string): Boolean;
begin
  try
    StrToInt(s);
    Result:= True;
    except
      Result:= False;
  end;
end;

begin
  Result:= True;
  List:= TStringList.Create;
  r:= StringReplace(r, #9, #13#10, [rfReplaceAll, rfIgnoreCase]);
  List.Text:= r;
  n:= List.Count;
  if (n = 0) then
    Result:= False
  else
  begin
    if (n <> fN) then
      Result:= False
    else
      for i:= 0 to n - 1 do
      begin
        s:= Trim(List[i]);
        if IsNum(s) then
        begin
          fValues[Col, i]:= StrToInt(s);
          if (Col > i) and (fValues[Col, i] <> fValues[i, Col]) then
          begin
            Result:= False;
            Break;
          end;
        end
        else
        begin
          Result:= False;
          Break;
        end;
      end;
  end;
  List.Free;
end;

function TGraph.ReadFileLn(var F: TextFile; out Ln: string): Boolean;
var
  ch: char;
  NewLn: string;
begin
  Result:= False;
  Ln:= '';
  Read(F, ch);
  while ((ch = #13) or (ch = #10)) do
    Read(F, ch);
  while ((ch <> #26) and (ch <> #13) and (ch <> #10)) do
  begin
    Ln:= Ln + ch;
    Read(F, ch);
    Result:= True;
  end;
  if (Trim(Ln) = '') then
  begin
    Result:= ReadFileLn(F, NewLn);
    Ln:= NewLn;
  end;
end;

function TGraph.LoadValuesFromFile(const FName: string): integer;
var
  F: TextFile;
  s: string;
  c: Integer;
begin
  Result:= E_NOERROR;
  AssignFile(F, FName);
  try
    FileMode:= fmOpenRead;
    Reset(F);
    c:= 0;
    while (not EOF(F)) do
    begin
      if (c = fN) then
      begin
        Result:= E_INVALIDFILE;
        Break;
      end;
      ReadFileLn(F, s);
      if (not AddRow(s, c)) then
      begin
        Result:= E_INVALIDFILE;
        Break;
      end;
      Inc(c);
    end;
    CloseFile(F);
    except
      Result:= E_FILEOPEN;
  end;
end;

function TGraph.GetValue(const i, j: Integer): Integer;
begin
  if ((i >= fN) or (j >= fN)) then
    Result:= 0
  else
    Result:= fValues[i, j];
end;

function TGraph.GetSize: Integer;
begin
  Result:= fN;
end;

procedure TGraph.SetSize(const Value: Integer);
begin
  fN:= Value;
  DestroyValues;
  CreateValues;
end;

function TGraph.SaveValuesToFile(const FName: string): integer;
var
  F: TextFile;
  i, j: Integer;
begin
  Result:= E_NOERROR;
  AssignFile(F, FName);
  try
    FileMode:=  fmOpenWrite;
    Rewrite(F);
    for i:= 0 to fN - 1 do
    begin
      if (i > 0) then
        Write(F, #13#10);
      for j:= 0 to fN - 1 do
      begin
        Write(F, fValues[i, j]);
        if (j < fN - 1) then
          Write(F, #9);
      end;
    end;
    CloseFile(F);
    except
      Result:= E_FILESAVE;
  end;
end;

end.
